﻿
Imports MONOW_PAL_VIEWER

Public Class ClsDataPayloadPalSensors
    Implements ITweDataPayload

    Dim _bStored As Boolean
    Dim _aElements() As ClsDataPalSenseElement

    Public Property Elements As ClsDataPalSenseElement()
        Get
            Return _aElements
        End Get
        Set(value As ClsDataPalSenseElement())
            _aElements = value
        End Set
    End Property

    ''' <summary>
    ''' Sensor Code による要素の抽出を行う
    ''' </summary>
    ''' <param name="snscd"></param>
    ''' <returns>リスト形式の要素</returns>
    Default Public ReadOnly Property SensorCode(ByVal snscd As UInteger) As List(Of ClsDataPalSenseElement)
        Get
            Dim objs = New List(Of ClsDataPalSenseElement)

            For Each e As ClsDataPalSenseElement In _aElements
                If e.SensorCode = snscd Then
                    objs.Add(e)
                End If
            Next

            Return objs
        End Get
    End Property

    Public ReadOnly Property IsStored As Boolean
        Get
            Return _bStored
        End Get
    End Property

    Public Function toByteArray() As Byte() Implements ITweDataPayload.toByteArray
        toByteArray = Nothing
    End Function

    Public Function parseByteArray(ByRef b() As Byte) As Boolean Implements ITweDataPayload.parseByteArray
        Dim i As Integer, j As Integer, bIdx As Integer
        Dim iLen As Integer = b(0)
        ReDim Elements(iLen - 1)

        Try
            bIdx = 1

            For i = 0 To iLen - 1
                Dim e As ClsDataPalSenseElement = New ClsDataPalSenseElement()

                Dim bDataType As Byte = b(bIdx)
                e.SensorCode = b(bIdx + 1)
                e.Ex = Nothing
                If bDataType And &H10 Then
                    e.Ex = New Byte
                    e.Ex = b(bIdx + 2)
                End If
                Dim bLen As Byte = b(bIdx + 3)

                bIdx += 4

                If (bDataType And &H80) = &H80 Then
                    ' Sensor Error
                    e.IsError = True
                    e.ErrorCode = bDataType And &H7F
                Else
                    ' Sensor is valid
                    e.IsError = False

                    Dim DatType As Integer = bDataType And &H3
                    Dim bDataCt As Byte

                    Select Case (bDataType And &H3)

                        Case 0 ' uchar
                            bDataCt = bLen - 1
                            e.Data = New Byte(bDataCt) {}
                            For j = 0 To bDataCt
                                e.Data(j) = b(bIdx + j)
                            Next

                        Case 4 ' ichar
                            bDataCt = bLen - 1
                            e.Data = New SByte(bDataCt) {}
                            For j = 0 To bDataCt
                                e.Data(j) = b(bIdx + j)
                            Next

                        Case 1 ' ushort
                            bDataCt = bLen / 2 - 1
                            e.Data = New UShort(bDataCt) {}
                            For j = 0 To bDataCt
                                e.Data(j) = get_BE_WORD(b, bIdx + j * 2)
                            Next

                        Case 5 ' short
                            bDataCt = bLen / 2 - 1
                            e.Data = New Short(bDataCt) {}
                            For j = 0 To bDataCt
                                Dim c As UShort
                                c = get_BE_WORD(b, bIdx + j * 2)
                                e.Data(j) = c.ToSigned
                            Next


                        Case 2 ' uint
                            bDataCt = bLen / 4 - 1
                            e.Data = New UInteger(bDataCt) {}
                            For j = 0 To bDataCt
                                e.Data(j) = get_BE_DWORD(b, bIdx + j * 4)
                            Next

                        Case 6 ' short
                            bDataCt = bLen / 4
                            e.Data = New Integer(bDataCt) {}
                            For j = 0 To bDataCt
                                Dim c As UShort
                                c = get_BE_DWORD(b, bIdx + j * 4)
                                e.Data(j) = c.ToSigned
                            Next

                        Case 7 ' 可変長
                            bDataCt = bLen - 1
                            e.Data = New Byte(bLen) {}
                            For j = 0 To bDataCt
                                e.Data(j) = b(bIdx + j)
                            Next

                    End Select

                End If

                Elements(i) = e

                bIdx += bLen
            Next

            _bStored = True
            parseByteArray = True
        Catch ex As Exception
            _bStored = False
            parseByteArray = False
        End Try

    End Function

    Public Function getCmdId() As Byte Implements ITweDataPayload.getCmdId
        getCmdId = &HFF
    End Function

    Public Sub New()
        clearData()
    End Sub

    Public Sub New(ByRef _b As Byte())
        parseByteArray(_b)

        If Not IsStored Then
            Throw New Exception
        End If
    End Sub

    Private Sub clearData()
        _bStored = False
    End Sub
End Class
