﻿Friend Module MDataGenerator

    Public Const twePayloadFmtTypeSimple As Byte = &H1
    Public Const twePayloadFmtTypePal As Byte = &H2

    ''' <summary>
    ''' コマンド番号に合致するペイロードオブジェクトを生成するジェネレータ(標準アプリ用)
    ''' </summary>
    ''' <param name="b"></param>
    ''' <param name="cmd"></param>
    ''' <returns>失敗時には Nothing を返す</returns>
    ''' <remarks></remarks>
    Function genDataPayloadStdApp(ByRef b As Byte(), ByVal cmd As Byte) As ITweDataPayload
        genDataPayloadStdApp = Nothing

        Select Case cmd
            Case ClsDataPayloadStdApp_0x80._BY_PKT_ID
                Try
                    genDataPayloadStdApp = New ClsDataPayloadStdApp_0x80(b)
                Catch ex As Exception
                    ' 解釈がエラー
                End Try

            Case ClsDataPayloadStdApp_0x81._BY_PKT_ID
                Try
                    genDataPayloadStdApp = New ClsDataPayloadStdApp_0x81(b)
                Catch ex As Exception
                    ' 解釈がエラー
                End Try

        End Select

    End Function


    ''' <summary>
    ''' コマンド番号に合致するペイロードオブジェクトを生成するジェネレータ (PAL フォーマット用)
    ''' </summary>
    ''' <param name="b"></param>
    ''' <returns>失敗時には Nothing を返す</returns>
    ''' <remarks></remarks>
    Function genDataPayloadPAL(ByRef b As Byte()) As ITweDataPayload
        ' PAL 形式のデータ解釈を行う
        genDataPayloadPAL = New ClsDataPayloadPalSensors(b)
    End Function

    ''' <summary>
    ''' バイト列から Format を判断し IDataFormat オブジェクトを生成する
    ''' ※ 本実装ではフォーマットは１種類のみなので  clsDataFormatSimple を生成している
    ''' </summary>
    ''' <param name="b"></param>
    ''' <returns>失敗時には Nothing を返す</returns>
    ''' <remarks></remarks>
    Function genDataFormat(ByRef b As Byte()) As ITweDataFormat
        genDataFormat = Nothing

        If b(0) < &HF0 Then
            Try
                ' genDataFormat = New ClsDataFormatStdApp(b) ' FOR STD APP (APP_TWELITE) -- NOT PRESENT IN THIS APP.
                genDataFormat = New ClsDataFormatPalApp(b) ' FOR PAL APP
            Catch ex As Exception
                ' 解釈がエラー
            End Try
            Exit Function
        End If
    End Function

    ''' <summary>
    ''' ITweDataPresentation 形式のジェネレータ
    ''' ※ 本実装では形式は１つであるため、clsDataPresentationAscii の空オブジェクトを生成している
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function genDataPresentation() As ITweDataPresentation
        Return New ClsDataPresentationAscii()
    End Function

    ''' <summary>
    ''' ITweDataPresentation 形式のジェネレータ
    ''' ※ 本実装では形式は１つであるため、clsDataPresentationAscii の空オブジェクトを生成している
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function genDataPresentation(ByRef b As Byte()) As ITweDataPresentation
        Return New ClsDataPresentationAscii(b)
    End Function


    ''' <summary>
    ''' 配列をidxから始まる４バイト取り出し３２ビット整数に変換する（ビッグエンディアン）
    ''' </summary>
    ''' <param name="b"></param>
    ''' <param name="idx"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function get_BE_DWORD(ByRef b As Byte(), ByVal idx As Integer) As UInteger
        Dim i0, i1, i2, i3 As UInteger

        i0 = b(idx)
        i1 = b(idx + 1)
        i2 = b(idx + 2)
        i3 = b(idx + 3)

        i0 = i0 << 24
        i1 = i1 << 16
        i2 = i2 << 8

        get_BE_DWORD = i0 + i1 + i2 + i3
    End Function

    ''' <summary>
    ''' 配列をidxから始まる２バイト取り出し１６ビット整数に変換する（ビッグエンディアン）
    ''' </summary>
    ''' <param name="b"></param>
    ''' <param name="idx"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function get_BE_WORD(ByRef b As Byte(), ByVal idx As Integer) As UShort
        Dim i0, i1 As UShort

        i0 = b(idx)
        i1 = b(idx + 1)

        i0 = i0 << 8

        get_BE_WORD = i0 + i1
    End Function

    Function conv_WORT_TO_SHORT(ByVal Number As UShort)
        If Number > 32768 Then 'negative number
            Return (65536 - Number) * -1
        Else
            Return Number
        End If
    End Function

    Function conv_DWORT_TO_Integer(ByVal Number As UInteger)
        If Number > &H80000000 Then 'negative number
            Return (&H100000000 - Number) * -1
        Else
            Return Number
        End If
    End Function

    ''' <summary>
    ''' BYTE配列に３２ビットのデータを格納する（ビッグエンディアン）
    ''' </summary>
    ''' <param name="num"></param>
    ''' <param name="b"></param>
    ''' <param name="idx"></param>
    ''' <remarks></remarks>
    Sub set_BE_DWORD(ByVal num As UInteger, ByRef b As Byte(), ByVal idx As Integer)
        b(idx + 0) = CByte((num >> 24) And &HFF)
        b(idx + 1) = CByte((num >> 16) And &HFF)
        b(idx + 2) = CByte((num >> 8) And &HFF)
        b(idx + 3) = CByte(num And &HFF)
    End Sub

    ''' <summary>
    ''' BYTE配列に１６ビットデータを格納する（ビッグエンディアン）
    ''' </summary>
    ''' <param name="num"></param>
    ''' <param name="b"></param>
    ''' <param name="idx"></param>
    ''' <remarks></remarks>
    Sub set_BE_WORD(ByVal num As UShort, ByRef b As Byte(), ByVal idx As Integer)
        b(idx + 0) = CByte((num >> 8) And &HFF)
        b(idx + 1) = CByte(num And &HFF)
    End Sub

End Module
